using System;
using System.IO;
using System.Configuration;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Data.Odbc;
using System.Data.OleDb;
using System.Data.Common;
using System.Drawing;
using System.Text;
using System.Transactions;
using System.Windows.Forms;
using IBM.Data.DB2;
using IBM.Data.DB2.iSeries;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;

namespace iDB2ProviderTester
{
    public partial class Form1 : Form
    {
        private int batchSize = 5;

        public Form1()
        {
            InitializeComponent();
            DBHelper.Initialize();
        }

        private void btnDaab_Click(object sender, EventArgs e)
        {
            
            try
            {
                Database db = DatabaseFactory.CreateDatabase();
                string sSql = "SELECT e.EMPNO, FIRSTNME, LASTNAME, WORKDEPT, PICTURE  FROM EMPLOYEE e, EMP_PHOTO p WHERE e.EMPNO=p.EMPNO AND (p.PHOTO_FORMAT = @format)";
                DbCommand cmd = db.GetSqlStringCommand(sSql);
                string format = "gif";
                db.AddParameter(cmd, "@format", DbType.String, format.Length, ParameterDirection.Input, false, 0, 0, "", DataRowVersion.Current, format);
                //db.AddInParameter(cmd, "@format", DbType.String, "gif");

                //ODBC provider with ? parm marker
                //string sSql = "SELECT e.EMPNO, FIRSTNME, LASTNAME, WORKDEPT FROM EMPLOYEE e WHERE e.WORKDEPT = ?";
                //string sSql = "SELECT e.EMPNO, FIRSTNME, LASTNAME, WORKDEPT FROM EMPLOYEE e WHERE e.WORKDEPT = @dept";
                //DbCommand cmd = db.GetSqlStringCommand(sSql);
                //string dept = "D11";
                //db.AddParameter(cmd, "@dept", DbType.String, dept.Length, ParameterDirection.Input, false, 0, 0, "DEPT", DataRowVersion.Current, dept);
                //db.AddInParameter(cmd, "@dept", DbType.String, dept);

                //cmd.CommandType = CommandType.Text;
                //db.DiscoverParameters(cmd);
                //db.SetParameterValue(cmd, "@dept", "D11");

                DataSet ds = db.ExecuteDataSet(cmd);

                this.displayResultsInDgv(ds);
            }
            catch (Exception ex)
            {
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
                
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnGeneric_Click(object sender, EventArgs e)
        {
            
            try
            {
                DbProviderFactory provider = DbProviderFactories.GetFactory( DBHelper.ProviderName);
                DbConnection con = provider.CreateConnection();
                con.ConnectionString = DBHelper.ConnectionString;


                //con.ConnectionString = @"DataSource=SE570A2;UserID=db2user;Password=test26t;DefaultCollection=DB2USER;Naming=SQL";
                
                DbCommand com = provider.CreateCommand();
                com.Connection = con;
                com.CommandText = "SELECT e.EMPNO, FIRSTNME, LASTNAME, WORKDEPT, PICTURE  FROM EMPLOYEE e, EMP_PHOTO p WHERE e.EMPNO=p.EMPNO AND (p.PHOTO_FORMAT = @format)";
                com.CommandType = CommandType.Text;
                /*
                DbParameter parm = com.CreateParameter();
                parm.ParameterName = "@format";
                parm.DbType = DbType.String;
                parm.Value = "gif";
                com.Parameters.Add(parm);
                */
                discoverParameters(com);
                com.Parameters["@format"].Value = "gif";

                DataSet ds = new DataSet();
                DbDataAdapter ad = provider.CreateDataAdapter();
                ad.SelectCommand = com;

                con.Open();
                ad.Fill(ds);
                con.Close();
                this.displayResultsInDgv(ds);
                
            }
            catch (Exception ex)
            {
                txtMessages.AppendText("!!! EXCEPTION:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText("Stack Trace:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.StackTrace);
                
            }
        }
        private void displayResultsInDgv(DataSet ds)
        {
            BindingSource bindingSource1 = new BindingSource();
            // Automatically generate the DataGridView columns.
            dgvResults.AutoGenerateColumns = true;

            // Set up the data source.
            bindingSource1.DataSource = ds.Tables[0];
            dgvResults.DataSource = bindingSource1;

            // Automatically resize the visible rows.
            //dgvResults.AutoSizeRowsMode =
            //    DataGridViewAutoSizeRowsMode.AllCellsExceptHeaders;

            // Set the DataGridView control's border.
            dgvResults.BorderStyle = BorderStyle.Fixed3D;

            // Put the cells in edit mode when user enters them.
            dgvResults.EditMode = DataGridViewEditMode.EditOnEnter;
        }
        private static void discoverParameters(DbCommand command)
        {

            if (command == null) throw new ArgumentNullException("command");
            if (command.CommandText == null) throw new ArgumentNullException("command text");
            if (command.Connection == null) throw new ArgumentNullException("connection");
          
            DbConnection connection = command.Connection;
            connection.Open();
            if (connection is iDB2Connection)
            {

                iDB2CommandBuilder.DeriveParameters((iDB2Command)command);

            }
            else if (connection is DB2Connection)
            {

                DB2CommandBuilder.DeriveParameters((DB2Command)command);

            }
            else if (connection is OdbcConnection)
            {

                OdbcCommandBuilder.DeriveParameters((OdbcCommand)command);

            }
            else if (connection is OleDbConnection)
            {

                OleDbCommandBuilder.DeriveParameters((OleDbCommand)command);

            }
            connection.Close();
            
            
        }

        private void btnBatchInsert_Click(object sender, EventArgs e)
        {
            try
            {
                //get the image file into a stream reader.
                /*
                string fileName = "..\\..\\thumb_coffee.gif";
                FileStream FilStr = new FileStream(fileName, FileMode.Open);
                BinaryReader BinRed = new BinaryReader(FilStr);
                byte[] picData = BinRed.ReadBytes((int)BinRed.BaseStream.Length);
                */
                DbProviderFactory provider = DbProviderFactories.GetFactory(DBHelper.ProviderName);
                DbConnection con = provider.CreateConnection();
                con.ConnectionString = DBHelper.ConnectionString;

                DbCommand com = provider.CreateCommand();
                com.Connection = con;
                //com.CommandText = "INSERT INTO COFFEES (COF_NAME, SUP_ID, PRICE, SALES, TOTAL, PICTURE) VALUES (@COF_NAME, @SUP_ID, @PRICE, @SALES, @TOTAL, @PICTURE)";
                com.CommandText = "INSERT INTO COFFEES (COF_NAME, SUP_ID, PRICE, SALES, TOTAL) VALUES (@COF_NAME, @SUP_ID, @PRICE, @SALES, @TOTAL)";
                com.CommandType = CommandType.Text;
                /*
                DbParameter parm = com.CreateParameter();
                parm.ParameterName = "@format";
                parm.DbType = DbType.String;
                parm.Value = "gif";
                com.Parameters.Add(parm);
                */
                discoverParameters(com);
                con.Open();
                for (int i = 1; i <= batchSize; i++)
                {

                    com.Parameters["@COF_NAME"].Value = "Super_Kona_" + i;
                    com.Parameters["@SUP_ID"].Value = 150;
                    com.Parameters["@PRICE"].Value = 9.95;
                    com.Parameters["@SALES"].Value = 1000;
                    com.Parameters["@TOTAL"].Value = 9950;
                    //com.Parameters["@PICTURE"].Value = picData;
                    batchInsert(com);
                }
                com.ExecuteNonQuery();
                con.Close();
                // 
                txtMessages.AppendText(batchSize + " rows were successfully inserted with a single INSERT statement.");
                txtMessages.AppendText(Environment.NewLine);

                showCoffees(provider, con);

            }
            catch (Exception ex)
            {
                txtMessages.AppendText("!!! EXCEPTION:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText("Stack Trace:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.StackTrace);

            }
        }

        private void showCoffees(DbProviderFactory provider, DbConnection con)
        {
            DataSet ds = new DataSet();
            DbDataAdapter ad = provider.CreateDataAdapter();
            DbCommand cmdSel = provider.CreateCommand();
            cmdSel.Connection = con;
            cmdSel.CommandText = "SELECT * FROM COFFEES";
            cmdSel.CommandType = CommandType.Text;
            ad.SelectCommand = cmdSel;

            con.Open();
            ad.Fill(ds);
            con.Close();
            this.displayResultsInDgv(ds);
        }
        private static void batchInsert(DbCommand command)
        {

            if (command == null) throw new ArgumentNullException("command");
            
            
            if (command is iDB2Command)
            {

                ((iDB2Command)command).AddBatch();

            }
 
            else 
            {

                throw new Exception("AddBatch not supported by the provider"); 

            }
            

        }

        private void btnDistributedTx_Click(object sender, EventArgs e)
        {
            try
            {
                DbProviderFactory provider = DbProviderFactories.GetFactory(DBHelper.ProviderName);
                                
                using (TransactionScope scope = new TransactionScope())
                {
                    txtMessages.AppendText("ISO LEVEL " + Transaction.Current.IsolationLevel);
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Type " + Transaction.Current.GetType());
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Status " + Transaction.Current.TransactionInformation.Status);
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Local Identifier " + Transaction.Current.TransactionInformation.LocalIdentifier);
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Distributed Identifier " + Transaction.Current.TransactionInformation.DistributedIdentifier);
                    txtMessages.AppendText(Environment.NewLine);


                    
                    //using (iDB2Connection conn2 = new iDB2Connection("Database=SAMPLE;User ID=jarek;Password=list15li;Server=JAREK:iDB2c_iDB2"))
                    using (DbConnection conn = provider.CreateConnection())
                    using (DbConnection conn2 = provider.CreateConnection())
                    {
                        conn.ConnectionString = DBHelper.ConnectionString;
                        conn.Open();
                        using (DbCommand cmd = conn.CreateCommand())
                        {
                            cmd.CommandText = "INSERT INTO COFFEES (COF_NAME, SUP_ID, PRICE, SALES, TOTAL) " + 
                                "VALUES ('Colombia Supreme Select', 200, 19.95, 10, 199.50)";

                            cmd.ExecuteNonQuery();
                        }

                        // Insert a second record against a connection inside same transaction
                        // iDB2 provider will auto enlist in the transaction if one exists - which is the case

                        conn2.ConnectionString = DBHelper.ConnectionString;
                        conn2.Open();
                        using (DbCommand cmd = conn2.CreateCommand())
                        {
                            cmd.CommandText = "INSERT INTO COFFEES (COF_NAME, SUP_ID, PRICE, SALES, TOTAL) VALUES ('Kenya Gold', 999, 14.95, 10, 149.50)";
                            cmd.ExecuteNonQuery();
                        }

                    }
                    txtMessages.AppendText("Type " + Transaction.Current.GetType());
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Status " + Transaction.Current.TransactionInformation.Status);
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Local Identifier " + Transaction.Current.TransactionInformation.LocalIdentifier);
                    txtMessages.AppendText(Environment.NewLine);
                    txtMessages.AppendText("Distributed Identifier " + Transaction.Current.TransactionInformation.DistributedIdentifier);
                    txtMessages.AppendText(Environment.NewLine);

         
                    scope.Complete(); //Beta 2 this should be .Complete
                    // 
                    txtMessages.AppendText("Distributed transaction has been successfully completed!.");
                    txtMessages.AppendText(Environment.NewLine);

                    
                 
                }
                DbConnection con = provider.CreateConnection();
                con.ConnectionString = DBHelper.ConnectionString;
                showCoffees(provider, con);

            }
            catch (TransactionAbortedException ex)
            {
                txtMessages.AppendText("!!! Transaction Aborted EXCEPTION:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
            }
            catch (Exception ex)
            {
                txtMessages.AppendText("!!! EXCEPTION:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText("Stack Trace:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.StackTrace);
            }
   
        }

        private void btnPurge_Click(object sender, EventArgs e)
        {
            try
            {
                DbProviderFactory provider = DbProviderFactories.GetFactory(DBHelper.ProviderName);
                DbConnection con = provider.CreateConnection();
                con.ConnectionString = DBHelper.ConnectionString;

                DbCommand com = provider.CreateCommand();
                com.Connection = con;
                com.CommandText = "DELETE FROM coffees WITH NC";
                com.CommandType = CommandType.Text;

                con.Open();
                com.ExecuteNonQuery();
                con.Close();
                // 
                txtMessages.AppendText("ALL rows were successfully deleted from COFFEES.");
                txtMessages.AppendText(Environment.NewLine);

                showCoffees(provider, con);


            }
            catch (Exception ex)
            {
                txtMessages.AppendText("!!! EXCEPTION:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.Message);
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText("Stack Trace:");
                txtMessages.AppendText(Environment.NewLine);
                txtMessages.AppendText(ex.StackTrace);

            } 
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }


        

    }
}